/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003-2004  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>

#include "ui.h"
#include "machine.h"
#include "snapshot.h"
#include "wd2797.h"
#include "vdg.h"
#include "video.h"
#include "fs.h"
#include "types.h"

#define FG MAPCOLOUR(255,255,255)
#define BG MAPCOLOUR(0,0,128)

void ui_init(void) {
}

static int dirsort(const void *a, const void *b) {
	FS_DIRENT *aa = (FS_DIRENT *)a;
	FS_DIRENT *bb = (FS_DIRENT *)b;
	if ((aa->attr & FS_ATTR_DIRECTORY) && !(bb->attr & FS_ATTR_DIRECTORY))
		return -1;
	if (!(aa->attr & FS_ATTR_DIRECTORY) && (bb->attr & FS_ATTR_DIRECTORY))
		return 1;
	return strncmp(aa->name, bb->name, 16);
}

static void draw_char(int x, int y, char c) {
	PIXEL *p = VIDEO_CURRENTBASE + (239-(y*8)) + x*6*240;
	Uint32 *g;
	Uint32 i;
	c &= 0x3f;
	g = &vdg_alpha[c*12+2];
	for (i = 0 ; i < 8; i++) {
		if (g[i] & 0x20) *p = FG;
		if (g[i] & 0x10) *(p+1*XSTEP) = FG;
		if (g[i] & 0x08) *(p+2*XSTEP) = FG;
		if (g[i] & 0x04) *(p+3*XSTEP) = FG;
		if (g[i] & 0x02) *(p+4*XSTEP) = FG;
		p--;
	}
}

static void draw_string(int x, int y, char *s, int w) {
	if (s == NULL)
		return;
	for (; *s && w; w--) {
		draw_char(x++,y,*(s++));
	}
}

static void highlight_line(int x, int y, int w) {
	PIXEL *p = VIDEO_CURRENTBASE + (231-(y*8)) + x*6*240;
	Uint32 i;
	for (i = 0; i < w*6; i++) {
		*(p++) ^= 0xff;
		*(p++) ^= 0xff;
		*(p++) ^= 0xff;
		*(p++) ^= 0xff;
		*(p++) ^= 0xff;
		*(p++) ^= 0xff;
		*(p++) ^= 0xff;
		*(p++) ^= 0xff;
		*(p++) ^= 0xff;
		p += 231;
	}
}

/* extract is either a function to pull a string out of data given an index
 * or NULL.  If NULL, data is treated as (char **) */

int menu_select(char *(*extract)(void *, int), void *data, int n, int x, int y, int w, int h) {
	Uint32 key, oldkey = ~GpKeyGet(), newkey;
	int base = 0, cur = 0, oldbase, oldcur, i;
	int step = h / 2;
	char *line;
	if (n < 1) return -1;
	video_fillrect(x*6, y*8, w*6, (h*8)+1, BG);
	for (i = 0; i < h && (base+i) < n; i++) {
		if (extract) {
			line = extract(data, base+i);
		} else {
			line = ((char **)data)[base+i];
		}
		draw_string(x, y+i, line, w);
	}
	highlight_line(x, y+(cur-base), w);
	do {
		Uint32 ticks = GpTickCountGet();
		key = GpKeyGet();
		newkey = key & oldkey;
		oldkey = ~key;
		oldcur = cur;
		oldbase = base;
		if (key & GPC_VK_UP) cur--;
		if (key & GPC_VK_DOWN) cur++;
		if (key & GPC_VK_FL) cur -= step;
		if (key & GPC_VK_FR) cur += step;
		if (cur < 0) cur = 0;
		if (cur >= n) cur = n - 1;
		base = cur - (cur % h);
		if (cur != oldcur) {
			highlight_line(x, y+(oldcur-oldbase), w);
			if (base != oldbase) {
				video_fillrect(x*6, y*8, w*6, h*8, BG);
				for (i = 0; i < h && (base+i) < n; i++) {
					draw_string(x, y+i, extract(data, base+i), w);
				}
			}
			highlight_line(x, y+(cur-base), w);
		}
		if (newkey & GPC_VK_START) {
			return -1;
		}
		while (GpTickCountGet()-ticks<450);
	} while (!(key & (GPC_VK_SELECT|GPC_VK_FB)));
	return cur;
}

static char *dirent_as_string(void *data, int n) {
	FS_DIR *dir = (FS_DIR *)data;
	static char result[16];
	result[0] = 0;
	if (n >= dir->num_files)
		return result;
	if (dir->dirents[n].attr & FS_ATTR_DIRECTORY) {
		strcpy(result, "<");
		strcat(result, dir->dirents[n].name);
		strcat(result, ">");
	} else {
		strcat(result, dir->dirents[n].name);
	}
	return result;
}

char *ui_get_filename(char *extension) {
	FS_DIR *dir, *olddir;
	int num, sel;
	static char result[16];
	dir = fs_scandir("gp:\\gpmm\\dragon", extension);
	while (1) {
		video_backup();
		num = dir->num_files;
		qsort(dir->dirents, num, sizeof(FS_DIRENT), dirsort);
		sel = menu_select(dirent_as_string, dir, num, 1,1,16,22);
		video_restore();
		if (sel == -1) {
			fs_freedir(dir);
			return NULL;
		}
		if (!(dir->dirents[sel].attr & FS_ATTR_DIRECTORY)) {
			memcpy(result, dir->dirents[sel].name, 16);
			fs_freedir(dir);
			return result;
		}
		olddir = dir;
		dir = fs_scandir(olddir->dirents[sel].name, extension);
		fs_freedir(olddir);
	}
}

void ui_menu(void) {
	int sel;
	char *next_rom[] = { "TANDY COCO ROM", "DRAGON ROM", };
	char *next_keymap[] = { "TANDY COCO KEYMAP", "DRAGON KEYMAP", };
	char *menu[] = {
		"LOAD SNAPSHOT",
		dragondos_enabled ? "DISABLE DRAGONDOS" : "ENABLE DRAGONDOS",
		"INSERT DISK",
		next_rom[machine_romtype],
		next_keymap[machine_keymap],
		"RESET DRAGON",
	};
	video_backup();
	sel = menu_select(NULL, (void *)menu, 6, 2,2,21,6);
	video_restore();
	switch(sel) {
		case -1: return; break;
		case 0: snapshot_state = 2; break;
		case 1: dragondos_enabled = !dragondos_enabled; break;
		case 2: wd2797_load_disk(ui_get_filename("VDK"), 0); break;
		case 3: machine_set_romtype(machine_romtype+1); snapshot_state = 3; break;
		case 4: machine_set_keymap(machine_keymap+1); break;
		case 5: snapshot_state = 3; break;
	}
}
